package commands

import (
	"encoding/json"
	"errors"
	"fmt"
	"io"

	"github.com/spf13/cobra"
	yaml "gopkg.in/yaml.v2"

	"github.com/brigadecore/brigade/pkg/storage/kube"
)

const projectGetUsage = `Get details for a project.

Print the attributes of a project.

The PROJECT may either be the ID or the name of a project.
`

func init() {
	project.AddCommand(projectGet)
	projectGet.Flags().StringVarP(&output, "output", "o", "", "Return output in another format. Supported formats: json")
}

var projectGet = &cobra.Command{
	Use:   "get PROJECT",
	Short: "get a project",
	Long:  projectGetUsage,
	RunE: func(cmd *cobra.Command, args []string) error {
		if len(args) == 0 {
			return errors.New("project name is a required argument")
		}
		return getProject(cmd.OutOrStdout(), args[0])
	},
}

func getProject(out io.Writer, name string) error {
	c, err := kubeClient()
	if err != nil {
		return err
	}

	store := kube.New(c, globalNamespace)
	p, err := store.GetProject(name)
	if err != nil {
		return err
	}

	sec, err := kube.SecretFromProject(p)
	if err != nil {
		return err
	}

	if output == "json" {
		b, err := json.MarshalIndent(sec, "", "    ")
		if err != nil {
			return err
		}
		_, err = out.Write(b)
		return err
	}

	bytes, err := yaml.Marshal(sec)
	if err != nil {
		return err
	}
	fmt.Fprintln(out, string(bytes))
	return nil
}
